import json
import os
from datetime import datetime
from typing import Dict

from flask import Blueprint, request, render_template, session, redirect, url_for
from loguru import logger
from sqlalchemy import create_engine
from sqlalchemy.orm import Session

from rtmp_worker.model.db_models import WordsStats
from rtmp_worker.src.main import DATABASE_FILE


class ControllerPages:
    blueprint = Blueprint("pages", __name__)

    @staticmethod
    @blueprint.route('/', methods=["POST", "GET"])
    def index():
        result = render_template(
            'pages/index.html'
        )

        return result

    @staticmethod
    @blueprint.route('/api', methods=["POST", "GET"])
    def api():
        engine = create_engine(f'sqlite:///{DATABASE_FILE}')
        result = []
        with Session(engine) as session:
            words_stats = session.query(WordsStats).order_by(WordsStats.count.desc()).limit(1000).all()
            words_stats = [it for it in words_stats if it.word and len(it.word.word) > 3]
            # utf-8 encoding
            words_stats = [it for it in words_stats if it.word.word.encode('utf-8')]

            remove_words = [
                'tieši', 'šobrīd', 'protams', 'tāpēc', 'tādēļ', 'tādējādi', 'tādā', 'tāda', 'tādi', 'tādas', 'tādu',
                'tiek', 'tāpēc', 'tāpat', 'tās', 'tā', 'tādēļ', 'tādējādi', 'tādā', 'tāda', 'tādi', 'tādas', 'tādu',
                'varbūt', 'tātad', 'mums', 'kādi', 'kāda', 'kādas', 'kādu', 'kāds', 'kā', 'kādēļ', 'kādēļ', 'kādēļ',
                'līdz', 'tikai', 'teikt', 'tā', 'tādēļ', 'tādējādi', 'tādā', 'tāda', 'tādi', 'tādas', 'tādu', 'tāpat',
                'bija', 'ļoti', 'tā', 'tādēļ', 'tādējādi', 'tādā', 'tāda', 'tādi', 'tādas', 'tādu', 'tāpat', 'tās',
                'kurā', 'kurš', 'kuram', 'kuru', 'viena', 'jums', 'jūsu', 'kaut', 'kāpēc', 'kāpēc', 'kāpēc', 'kāpēc',
                'vispār', 'viņiem', 'viņu', 'viņš', 'viņa', 'viņas', 'viņi', 'viņu', 'viņš', 'viņa', 'viņas', 'viņi',
                'visticamāk', 'vienkārši', 'vienīgi', 'vienā', 'vienas', 'vieni', 'vienus', 'vienā', 'vienas', 'vieni',
                'vieniem', 'vienus', 'vienā', 'vienas', 'vieni', 'vieniem', 'vienus', 'vienā', 'vienas', 'vieni', 'vieniem',
                'vienotus', 'vienā', 'vienas', 'vieni', 'vieniem', 'vienus', 'vienā', 'vienas', 'vieni', 'vieniem', 'vienus',
                'sanāk', 'tā', 'tādēļ', 'tādējādi', 'tādā', 'tāda', 'tādi', 'tādas', 'tādu', 'tāpat', 'tās', 'tā',
                'viņus', 'viņiem', 'viņu', 'viņš', 'viņa', 'viņas', 'viņi', 'viņu', 'viņš', 'viņa', 'viņas', 'viņi',
                'lietās', 'lietām', 'lietā', 'lieta', 'lietas', 'lietās', 'lietām', 'lietā', 'lieta', 'lietas', 'lietās',
                'saviem', 'kurai', 'sakot', 'varēs', 'varētu', 'varēja', 'varējis', 'varējām', 'varējāt', 'varējuši',
                'linda', 'esmu', 'šāda', 'šādi', 'šādas', 'šādu', 'šādā', 'šādās', 'šādiem', 'šādos', 'šādus', 'šādām',
                'skaita', 'kura', 'darāt', 'darīt', 'darīja', 'darījis', 'darījām', 'darījāt', 'darījuši', 'darījās',
                'dažiem', 'dažus', 'dažām', 'dažās', 'dažā', 'daži', 'dažām', 'dažās', 'dažā', 'daži', 'dažiem', 'dažus',
                'citā', 'savus', 'viņam', 'savos', 'gadus', 'daļa', 'kurus', 'sauc', 'pašām', 'vajag', 'vajadzētu',
                'dabūt', 'citus', 'pašai', 'pašas', 'paši', 'pašu', 'pašā', 'pašās', 'pašiem', 'pašos', 'pašus', 'pašām',
                'galu', 'sākuma', 'gati', 'citi',
            ]
            words_stats = [it for it in words_stats if it.word.word.lower() not in remove_words]

            idx = 0
            while idx < len(words_stats):
                word_each = words_stats[idx]
                word_stem = word_each.word.word[:-2]
                is_found = False
                for idx_2 in range(idx + 1, len(words_stats)):
                    word_other = words_stats[idx_2]
                    len_dif = len(word_other.word.word) - len(word_stem)
                    if len_dif <= 3:
                        if word_other.word.word.startswith(word_stem):
                            is_found = True
                            word_other.count += word_each.count
                if is_found:
                    words_stats.remove(word_each)
                else:
                    idx += 1

            top_words_stats = list(words_stats)
            # sort by count
            top_words_stats.sort(key=lambda x: x.count, reverse=True)
            top_words_stats = top_words_stats[:40]
            total_words = 0
            for each in top_words_stats:
                total_words += each.count
            for each in top_words_stats:
                result.append({
                    'text': each.word.word,
                    'count': each.count,
                    'weight': round(each.count / total_words * 100, 2)
                })

        return json.dumps(result, indent=4)


